@@ -1,13 +1,14 @@
 # util.py
+import inspect
 import warnings
 import types
 import collections
 import itertools
-from functools import lru_cache
-from typing import List
-
+from functools import lru_cache, wraps
+from typing import Callable, List, Union, Iterable, TypeVar, cast
 
 _bslash = chr(92)
+C = TypeVar("C", bound=Callable)
 
 
 class __config_flags:
@@ -21,32 +22,29 @@ class __config_flags:
     def _set(cls, dname, value):
         if dname in cls._fixed_names:
             warnings.warn(
-                "{}.{} {} is {} and cannot be overridden".format(
-                    cls.__name__,
-                    dname,
-                    cls._type_desc,
-                    str(getattr(cls, dname)).upper(),
-                )
+                f"{cls.__name__}.{dname} {cls._type_desc} is {str(getattr(cls, dname)).upper()}"
+                f" and cannot be overridden",
+                stacklevel=3,
             )
             return
         if dname in cls._all_names:
             setattr(cls, dname, value)
         else:
-            raise ValueError("no such {} {!r}".format(cls._type_desc, dname))
+            raise ValueError(f"no such {cls._type_desc} {dname!r}")
 
     enable = classmethod(lambda cls, name: cls._set(name, True))
     disable = classmethod(lambda cls, name: cls._set(name, False))
 
 
 @lru_cache(maxsize=128)
-def col(loc: int, strg: str):
+def col(loc: int, strg: str) -> int:
     """
     Returns current column within a string, counting newlines as line separators.
     The first column is number 1.
 
     Note: the default parsing behavior is to expand tabs in the input string
     before starting the parsing process.  See
-    :class:`ParserElement.parseString` for more
+    :class:`ParserElement.parse_string` for more
     information on parsing strings containing ``<TAB>`` s, and suggested
     methods to maintain a consistent view of the parsed string, the parse
     location, and line and column positions within the parsed string.
@@ -56,12 +54,12 @@ def col(loc: int, strg: str):
 
 
 @lru_cache(maxsize=128)
-def lineno(loc: int, strg: str):
+def lineno(loc: int, strg: str) -> int:
     """Returns current line number within a string, counting newlines as line separators.
     The first line is number 1.
 
     Note - the default parsing behavior is to expand tabs in the input string
-    before starting the parsing process.  See :class:`ParserElement.parseString`
+    before starting the parsing process.  See :class:`ParserElement.parse_string`
     for more information on parsing strings containing ``<TAB>`` s, and
     suggested methods to maintain a consistent view of the parsed string, the
     parse location, and line and column positions within the parsed string.
@@ -70,13 +68,13 @@ def lineno(loc: int, strg: str):
 
 
 @lru_cache(maxsize=128)
-def line(loc: int, strg: str):
+def line(loc: int, strg: str) -> str:
     """
     Returns the line of text containing loc within a string, counting newlines as line separators.
     """
-    lastCR = strg.rfind("\n", 0, loc)
-    nextCR = strg.find("\n", loc)
-    return strg[lastCR + 1 : nextCR] if nextCR >= 0 else strg[lastCR + 1 :]
+    last_cr = strg.rfind("\n", 0, loc)
+    next_cr = strg.find("\n", loc)
+    return strg[last_cr + 1 : next_cr] if next_cr >= 0 else strg[last_cr + 1 :]
 
 
 class _UnboundedCache:
@@ -85,41 +83,46 @@ class _UnboundedCache:
         cache_get = cache.get
         self.not_in_cache = not_in_cache = object()
 
-        def get(self, key):
+        def get(_, key):
             return cache_get(key, not_in_cache)
 
-        def set(self, key, value):
+        def set_(_, key, value):
             cache[key] = value
 
-        def clear(self):
+        def clear(_):
             cache.clear()
 
         self.size = None
         self.get = types.MethodType(get, self)
-        self.set = types.MethodType(set, self)
+        self.set = types.MethodType(set_, self)
         self.clear = types.MethodType(clear, self)
 
 
 class _FifoCache:
     def __init__(self, size):
         self.not_in_cache = not_in_cache = object()
-        cache = collections.OrderedDict()
+        cache = {}
+        keyring = [object()] * size
         cache_get = cache.get
+        cache_pop = cache.pop
+        keyiter = itertools.cycle(range(size))
 
-        def get(self, key):
+        def get(_, key):
             return cache_get(key, not_in_cache)
 
-        def set(self, key, value):
+        def set_(_, key, value):
             cache[key] = value
-            while len(cache) > size:
-                cache.popitem(last=False)
+            i = next(keyiter)
+            cache_pop(keyring[i], None)
+            keyring[i] = key
 
-        def clear(self):
+        def clear(_):
             cache.clear()
+            keyring[:] = [object()] * size
 
         self.size = size
         self.get = types.MethodType(get, self)
-        self.set = types.MethodType(set, self)
+        self.set = types.MethodType(set_, self)
         self.clear = types.MethodType(clear, self)
 
 
@@ -171,7 +174,7 @@ class UnboundedMemo(dict):
         pass
 
 
-def _escapeRegexRangeChars(s: str):
+def _escape_regex_range_chars(s: str) -> str:
     # escape these chars: ^-[]
     for c in r"\^-[]":
         s = s.replace(c, _bslash + c)
@@ -180,7 +183,9 @@ def _escapeRegexRangeChars(s: str):
     return str(s)
 
 
-def _collapseStringToRanges(s: str, re_escape: bool = True):
+def _collapse_string_to_ranges(
+    s: Union[str, Iterable[str]], re_escape: bool = True
+) -> str:
     def is_consecutive(c):
         c_int = ord(c)
         is_consecutive.prev, prev = c_int, is_consecutive.prev
@@ -188,9 +193,9 @@ def _collapseStringToRanges(s: str, re_escape: bool = True):
             is_consecutive.value = next(is_consecutive.counter)
         return is_consecutive.value
 
-    is_consecutive.prev = 0
-    is_consecutive.counter = itertools.count()
-    is_consecutive.value = -1
+    is_consecutive.prev = 0  # type: ignore [attr-defined]
+    is_consecutive.counter = itertools.count()  # type: ignore [attr-defined]
+    is_consecutive.value = -1  # type: ignore [attr-defined]
 
     def escape_re_range_char(c):
         return "\\" + c if c in r"\^-][" else c
@@ -212,10 +217,9 @@ def _collapseStringToRanges(s: str, re_escape: bool = True):
             if first == last:
                 ret.append(escape_re_range_char(first))
             else:
+                sep = "" if ord(last) == ord(first) + 1 else "-"
                 ret.append(
-                    "{}-{}".format(
-                        escape_re_range_char(first), escape_re_range_char(last)
-                    )
+                    f"{escape_re_range_char(first)}{sep}{escape_re_range_char(last)}"
                 )
     else:
         ret = [escape_re_range_char(c) for c in s]
@@ -223,7 +227,7 @@ def _collapseStringToRanges(s: str, re_escape: bool = True):
     return "".join(ret)
 
 
-def _flatten(ll: List):
+def _flatten(ll: list) -> list:
     ret = []
     for i in ll:
         if isinstance(i, list):
@@ -231,3 +235,43 @@ def _flatten(ll: List):
         else:
             ret.append(i)
     return ret
+
+
+def replaced_by_pep8(compat_name: str, fn: C) -> C:
+    # In a future version, uncomment the code in the internal _inner() functions
+    # to begin emitting DeprecationWarnings.
+
+    # Unwrap staticmethod/classmethod
+    fn = getattr(fn, "__func__", fn)
+
+    # (Presence of 'self' arg in signature is used by explain_exception() methods, so we take
+    # some extra steps to add it if present in decorated function.)
+    if "self" == list(inspect.signature(fn).parameters)[0]:
+
+        @wraps(fn)
+        def _inner(self, *args, **kwargs):
+            # warnings.warn(
+            #     f"Deprecated - use {fn.__name__}", DeprecationWarning, stacklevel=2
+            # )
+            return fn(self, *args, **kwargs)
+
+    else:
+
+        @wraps(fn)
+        def _inner(*args, **kwargs):
+            # warnings.warn(
+            #     f"Deprecated - use {fn.__name__}", DeprecationWarning, stacklevel=2
+            # )
+            return fn(*args, **kwargs)
+
+    _inner.__doc__ = f"""Deprecated - use :class:`{fn.__name__}`"""
+    _inner.__name__ = compat_name
+    _inner.__annotations__ = fn.__annotations__
+    if isinstance(fn, types.FunctionType):
+        _inner.__kwdefaults__ = fn.__kwdefaults__
+    elif isinstance(fn, type) and hasattr(fn, "__init__"):
+        _inner.__kwdefaults__ = fn.__init__.__kwdefaults__
+    else:
+        _inner.__kwdefaults__ = None
+    _inner.__qualname__ = fn.__qualname__
+    return cast(C, _inner)
